/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.rxjava.cassandra;

import rx.Observable;
import rx.Single;
import io.vertx.rx.java.RxHelper;
import io.vertx.rx.java.WriteStreamSubscriber;
import io.vertx.rx.java.SingleOnSubscribeAdapter;
import java.util.Map;
import java.util.Set;
import java.util.List;
import java.util.Iterator;
import java.util.function.Function;
import java.util.stream.Collectors;
import io.vertx.core.Handler;
import io.vertx.core.AsyncResult;
import io.vertx.core.json.JsonObject;
import io.vertx.core.json.JsonArray;
import io.vertx.lang.rx.RxGen;
import io.vertx.lang.rx.TypeArg;
import io.vertx.lang.rx.MappingIterator;

/**
 * It is like , but adapted for Vert.x.
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.cassandra.Mapper original} non RX-ified interface using Vert.x codegen.
 */

@RxGen(io.vertx.cassandra.Mapper.class)
public class Mapper<T> {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    Mapper that = (Mapper) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final TypeArg<Mapper> __TYPE_ARG = new TypeArg<>(    obj -> new Mapper((io.vertx.cassandra.Mapper) obj),
    Mapper::getDelegate
  );

  private final io.vertx.cassandra.Mapper<T> delegate;
  public final TypeArg<T> __typeArg_0;
  
  public Mapper(io.vertx.cassandra.Mapper delegate) {
    this.delegate = delegate;
    this.__typeArg_0 = TypeArg.unknown();  }

  public Mapper(Object delegate, TypeArg<T> typeArg_0) {
    this.delegate = (io.vertx.cassandra.Mapper)delegate;
    this.__typeArg_0 = typeArg_0;
  }

  public io.vertx.cassandra.Mapper getDelegate() {
    return delegate;
  }

  /**
   * Asynchronous save method.
   * @param entity object to be stored in database
   * @param handler result handler
   */
  public void save(T entity, Handler<AsyncResult<Void>> handler) { 
    delegate.save(__typeArg_0.<T>unwrap(entity), handler);
  }

    /**
   * Asynchronous save method.
   * @param entity object to be stored in database
   * @return 
   */
  public Single<Void> rxSave(T entity) { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      save(entity, fut);
    }));
  }

  /**
   * Asynchronous delete method based on the column values of the primary key.
   * @param primaryKey primary key used to find row to delete
   * @param handler result handler
   */
  public void delete(List<java.lang.Object> primaryKey, Handler<AsyncResult<Void>> handler) { 
    delegate.delete(primaryKey, handler);
  }

    /**
   * Asynchronous delete method based on the column values of the primary key.
   * @param primaryKey primary key used to find row to delete
   * @return 
   */
  public Single<Void> rxDelete(List<java.lang.Object> primaryKey) { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      delete(primaryKey, fut);
    }));
  }

  /**
   * Asynchronous get method based on the column values of the primary key.
   * @param primaryKey primary key used to retrieve row
   * @param handler result handler
   */
  public void get(List<java.lang.Object> primaryKey, Handler<AsyncResult<T>> handler) { 
    delegate.get(primaryKey, new Handler<AsyncResult<T>>() {
      public void handle(AsyncResult<T> ar) {
        if (ar.succeeded()) {
          handler.handle(io.vertx.core.Future.succeededFuture((T)__typeArg_0.wrap(ar.result())));
        } else {
          handler.handle(io.vertx.core.Future.failedFuture(ar.cause()));
        }
      }
    });
  }

    /**
   * Asynchronous get method based on the column values of the primary key.
   * @param primaryKey primary key used to retrieve row
   * @return 
   */
  public Single<T> rxGet(List<java.lang.Object> primaryKey) { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      get(primaryKey, fut);
    }));
  }

  public static <T> Mapper<T> newInstance(io.vertx.cassandra.Mapper arg) {
    return arg != null ? new Mapper<T>(arg) : null;
  }

  public static <T> Mapper<T> newInstance(io.vertx.cassandra.Mapper arg, TypeArg<T> __typeArg_T) {
    return arg != null ? new Mapper<T>(arg, __typeArg_T) : null;
  }

}
