/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.rxjava.core;

import rx.Observable;
import rx.Single;
import io.vertx.rx.java.RxHelper;
import io.vertx.rx.java.WriteStreamSubscriber;
import io.vertx.rx.java.SingleOnSubscribeAdapter;
import java.util.Map;
import java.util.Set;
import java.util.List;
import java.util.Iterator;
import java.util.function.Function;
import java.util.stream.Collectors;
import io.vertx.core.Handler;
import io.vertx.core.AsyncResult;
import io.vertx.core.json.JsonObject;
import io.vertx.core.json.JsonArray;
import io.vertx.lang.rx.RxGen;
import io.vertx.lang.rx.TypeArg;
import io.vertx.lang.rx.MappingIterator;

/**
 * Represents the result of an action that may, or may not, have occurred yet.
 * <p>
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.core.Future original} non RX-ified interface using Vert.x codegen.
 */

@RxGen(io.vertx.core.Future.class)
public class Future<T> {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    Future that = (Future) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final TypeArg<Future> __TYPE_ARG = new TypeArg<>(    obj -> new Future((io.vertx.core.Future) obj),
    Future::getDelegate
  );

  private final io.vertx.core.Future<T> delegate;
  public final TypeArg<T> __typeArg_0;
  
  public Future(io.vertx.core.Future delegate) {
    this.delegate = delegate;
    this.__typeArg_0 = TypeArg.unknown();  }

  public Future(Object delegate, TypeArg<T> typeArg_0) {
    this.delegate = (io.vertx.core.Future)delegate;
    this.__typeArg_0 = typeArg_0;
  }

  public io.vertx.core.Future getDelegate() {
    return delegate;
  }


  /**
   * Create a future that hasn't completed yet and that is passed to the <code>handler</code> before it is returned.
   * @param handler the handler
   * @return the future.
   */
  public static <T> io.vertx.rxjava.core.Future<T> future(Handler<io.vertx.rxjava.core.Promise<T>> handler) { 
    io.vertx.rxjava.core.Future<T> ret = io.vertx.rxjava.core.Future.newInstance((io.vertx.core.Future)io.vertx.core.Future.future(new Handler<io.vertx.core.Promise<T>>() {
      public void handle(io.vertx.core.Promise<T> event) {
        handler.handle(io.vertx.rxjava.core.Promise.newInstance((io.vertx.core.Promise)event, TypeArg.unknown()));
      }
    }), TypeArg.unknown());
    return ret;
  }

  /**
   * Create a future that hasn't completed yet
   * @return the future
   */
  @Deprecated()
  public static <T> io.vertx.rxjava.core.Future<T> future() { 
    io.vertx.rxjava.core.Future<T> ret = io.vertx.rxjava.core.Future.newInstance((io.vertx.core.Future)io.vertx.core.Future.future(), TypeArg.unknown());
    return ret;
  }

  /**
   * Create a succeeded future with a null result
   * @return the future
   */
  public static <T> io.vertx.rxjava.core.Future<T> succeededFuture() { 
    io.vertx.rxjava.core.Future<T> ret = io.vertx.rxjava.core.Future.newInstance((io.vertx.core.Future)io.vertx.core.Future.succeededFuture(), TypeArg.unknown());
    return ret;
  }

  /**
   * Created a succeeded future with the specified result.
   * @param result the result
   * @return the future
   */
  public static <T> io.vertx.rxjava.core.Future<T> succeededFuture(T result) { 
    io.vertx.rxjava.core.Future<T> ret = io.vertx.rxjava.core.Future.newInstance((io.vertx.core.Future)io.vertx.core.Future.succeededFuture(result), TypeArg.unknown());
    return ret;
  }

  /**
   * Create a failed future with the specified failure cause.
   * @param t the failure cause as a Throwable
   * @return the future
   */
  public static <T> io.vertx.rxjava.core.Future<T> failedFuture(java.lang.Throwable t) { 
    io.vertx.rxjava.core.Future<T> ret = io.vertx.rxjava.core.Future.newInstance((io.vertx.core.Future)io.vertx.core.Future.failedFuture(t), TypeArg.unknown());
    return ret;
  }

  /**
   * Create a failed future with the specified failure message.
   * @param failureMessage the failure message
   * @return the future
   */
  public static <T> io.vertx.rxjava.core.Future<T> failedFuture(String failureMessage) { 
    io.vertx.rxjava.core.Future<T> ret = io.vertx.rxjava.core.Future.newInstance((io.vertx.core.Future)io.vertx.core.Future.failedFuture(failureMessage), TypeArg.unknown());
    return ret;
  }

  /**
   * Has the future completed?
   * <p>
   * It's completed if it's either succeeded or failed.
   * @return true if completed, false if not
   */
  public boolean isComplete() { 
    boolean ret = delegate.isComplete();
    return ret;
  }

  /**
   * Like {@link io.vertx.rxjava.core.Future#onComplete}.
   * @param handler 
   * @return 
   */
  @Deprecated()
  public io.vertx.rxjava.core.Future<T> setHandler(Handler<AsyncResult<T>> handler) { 
    delegate.setHandler(new Handler<AsyncResult<T>>() {
      public void handle(AsyncResult<T> ar) {
        if (ar.succeeded()) {
          handler.handle(io.vertx.core.Future.succeededFuture((T)__typeArg_0.wrap(ar.result())));
        } else {
          handler.handle(io.vertx.core.Future.failedFuture(ar.cause()));
        }
      }
    });
    return this;
  }

    /**
   * Like {@link io.vertx.rxjava.core.Future#onComplete}.
   * @return 
   * @deprecated use {@link #rxSetHandler} instead
   */
  @Deprecated()
  public Observable<T> setHandlerObservable() { 
    io.vertx.rx.java.ObservableFuture<T> handler = io.vertx.rx.java.RxHelper.observableFuture();
    setHandler(handler.toHandler());
    return handler;
  }

  /**
   * Like {@link io.vertx.rxjava.core.Future#onComplete}.
   * @return 
   */
  @Deprecated()
  public Single<T> rxSetHandler() { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      setHandler(fut);
    }));
  }

  /**
   * Add a handler to be notified of the result.
   * <br/>
   * @param handler the handler that will be called with the result
   * @return a reference to this, so it can be used fluently
   */
  public io.vertx.rxjava.core.Future<T> onComplete(Handler<AsyncResult<T>> handler) { 
    delegate.onComplete(new Handler<AsyncResult<T>>() {
      public void handle(AsyncResult<T> ar) {
        if (ar.succeeded()) {
          handler.handle(io.vertx.core.Future.succeededFuture((T)__typeArg_0.wrap(ar.result())));
        } else {
          handler.handle(io.vertx.core.Future.failedFuture(ar.cause()));
        }
      }
    });
    return this;
  }

    /**
   * Add a handler to be notified of the result.
   * <br/>
   * @return a reference to this, so it can be used fluently
   * @deprecated use {@link #rxOnComplete} instead
   */
  @Deprecated()
  public Observable<T> onCompleteObservable() { 
    io.vertx.rx.java.ObservableFuture<T> handler = io.vertx.rx.java.RxHelper.observableFuture();
    onComplete(handler.toHandler());
    return handler;
  }

  /**
   * Add a handler to be notified of the result.
   * <br/>
   * @return a reference to this, so it can be used fluently
   */
  public Single<T> rxOnComplete() { 
    return Single.create(new io.vertx.rx.java.SingleOnSubscribeAdapter<>(fut -> {
      onComplete(fut);
    }));
  }

  /**
   * Add a handler to be notified of the succeeded result.
   * <br/>
   * @param handler the handler that will be called with the succeeded result
   * @return a reference to this, so it can be used fluently
   */
  public io.vertx.rxjava.core.Future<T> onSuccess(Handler<T> handler) { 
    delegate.onSuccess(new Handler<T>() {
      public void handle(T event) {
        handler.handle((T)__typeArg_0.wrap(event));
      }
    });
    return this;
  }

  /**
   * Add a handler to be notified of the failed result.
   * <br/>
   * @param handler the handler that will be called with the failed result
   * @return a reference to this, so it can be used fluently
   */
  public io.vertx.rxjava.core.Future<T> onFailure(Handler<java.lang.Throwable> handler) { 
    delegate.onFailure(handler);
    return this;
  }

  /**
   * Set the result. Any handler will be called, if there is one, and the future will be marked as completed.
   * @param result the result
   */
  @Deprecated()
  public void complete(T result) { 
    delegate.complete(__typeArg_0.<T>unwrap(result));
  }

  /**
   * Set a null result. Any handler will be called, if there is one, and the future will be marked as completed.
   */
  @Deprecated()
  public void complete() { 
    delegate.complete();
  }

  /**
   * Set the failure. Any handler will be called, if there is one, and the future will be marked as completed.
   * @param cause the failure cause
   */
  @Deprecated()
  public void fail(java.lang.Throwable cause) { 
    delegate.fail(cause);
  }

  /**
   * Try to set the failure. When it happens, any handler will be called, if there is one, and the future will be marked as completed.
   * @param failureMessage the failure message
   */
  @Deprecated()
  public void fail(String failureMessage) { 
    delegate.fail(failureMessage);
  }

  /**
   * Set the failure. Any handler will be called, if there is one, and the future will be marked as completed.
   * @param result the result
   * @return false when the future is already completed
   */
  @Deprecated()
  public boolean tryComplete(T result) { 
    boolean ret = delegate.tryComplete(__typeArg_0.<T>unwrap(result));
    return ret;
  }

  /**
   * Try to set the result. When it happens, any handler will be called, if there is one, and the future will be marked as completed.
   * @return false when the future is already completed
   */
  @Deprecated()
  public boolean tryComplete() { 
    boolean ret = delegate.tryComplete();
    return ret;
  }

  /**
   * Try to set the failure. When it happens, any handler will be called, if there is one, and the future will be marked as completed.
   * @param cause the failure cause
   * @return false when the future is already completed
   */
  @Deprecated()
  public boolean tryFail(java.lang.Throwable cause) { 
    boolean ret = delegate.tryFail(cause);
    return ret;
  }

  /**
   * Try to set the failure. When it happens, any handler will be called, if there is one, and the future will be marked as completed.
   * @param failureMessage the failure message
   * @return false when the future is already completed
   */
  @Deprecated()
  public boolean tryFail(String failureMessage) { 
    boolean ret = delegate.tryFail(failureMessage);
    return ret;
  }

  /**
   * The result of the operation. This will be null if the operation failed.
   * @return the result or null if the operation failed.
   */
  public T result() { 
    T ret = (T)__typeArg_0.wrap(delegate.result());
    return ret;
  }

  /**
   * A Throwable describing failure. This will be null if the operation succeeded.
   * @return the cause or null if the operation succeeded.
   */
  public java.lang.Throwable cause() { 
    java.lang.Throwable ret = delegate.cause();
    return ret;
  }

  /**
   * Did it succeed?
   * @return true if it succeded or false otherwise
   */
  public boolean succeeded() { 
    boolean ret = delegate.succeeded();
    return ret;
  }

  /**
   * Did it fail?
   * @return true if it failed or false otherwise
   */
  public boolean failed() { 
    boolean ret = delegate.failed();
    return ret;
  }

  /**
   * Alias for {@link io.vertx.rxjava.core.Future#compose}.
   * @param mapper 
   * @return 
   */
  public <U> io.vertx.rxjava.core.Future<U> flatMap(Function<T, io.vertx.rxjava.core.Future<U>> mapper) { 
    io.vertx.rxjava.core.Future<U> ret = io.vertx.rxjava.core.Future.newInstance((io.vertx.core.Future)delegate.flatMap(new Function<T,io.vertx.core.Future<U>>() {
      public io.vertx.core.Future<U> apply(T arg) {
        io.vertx.rxjava.core.Future<U> ret = mapper.apply((T)__typeArg_0.wrap(arg));
        return ret.getDelegate();
      }
    }), TypeArg.unknown());
    return ret;
  }

  /**
   * Compose this future with a <code>mapper</code> function.<p>
   * <p>
   * When this future (the one on which <code>compose</code> is called) succeeds, the <code>mapper</code> will be called with
   * the completed value and this mapper returns another future object. This returned future completion will complete
   * the future returned by this method call.<p>
   * <p>
   * If the <code>mapper</code> throws an exception, the returned future will be failed with this exception.<p>
   * <p>
   * When this future fails, the failure will be propagated to the returned future and the <code>mapper</code>
   * will not be called.
   * @param mapper the mapper function
   * @return the composed future
   */
  public <U> io.vertx.rxjava.core.Future<U> compose(Function<T, io.vertx.rxjava.core.Future<U>> mapper) { 
    io.vertx.rxjava.core.Future<U> ret = io.vertx.rxjava.core.Future.newInstance((io.vertx.core.Future)delegate.compose(new Function<T,io.vertx.core.Future<U>>() {
      public io.vertx.core.Future<U> apply(T arg) {
        io.vertx.rxjava.core.Future<U> ret = mapper.apply((T)__typeArg_0.wrap(arg));
        return ret.getDelegate();
      }
    }), TypeArg.unknown());
    return ret;
  }

  /**
   * Compose this future with a <code>successMapper</code> and <code>failureMapper</code> functions.<p>
   * <p>
   * When this future (the one on which <code>compose</code> is called) succeeds, the <code>successMapper</code> will be called with
   * the completed value and this mapper returns another future object. This returned future completion will complete
   * the future returned by this method call.<p>
   * <p>
   * When this future (the one on which <code>compose</code> is called) fails, the <code>failureMapper</code> will be called with
   * the failure and this mapper returns another future object. This returned future completion will complete
   * the future returned by this method call.<p>
   * <p>
   * If any mapper function throws an exception, the returned future will be failed with this exception.<p>
   * @param successMapper the function mapping the success
   * @param failureMapper the function mapping the failure
   * @return the composed future
   */
  public <U> io.vertx.rxjava.core.Future<U> compose(Function<T, io.vertx.rxjava.core.Future<U>> successMapper, Function<java.lang.Throwable, io.vertx.rxjava.core.Future<U>> failureMapper) { 
    io.vertx.rxjava.core.Future<U> ret = io.vertx.rxjava.core.Future.newInstance((io.vertx.core.Future)delegate.compose(new Function<T,io.vertx.core.Future<U>>() {
      public io.vertx.core.Future<U> apply(T arg) {
        io.vertx.rxjava.core.Future<U> ret = successMapper.apply((T)__typeArg_0.wrap(arg));
        return ret.getDelegate();
      }
    }, new Function<java.lang.Throwable,io.vertx.core.Future<U>>() {
      public io.vertx.core.Future<U> apply(java.lang.Throwable arg) {
        io.vertx.rxjava.core.Future<U> ret = failureMapper.apply(arg);
        return ret.getDelegate();
      }
    }), TypeArg.unknown());
    return ret;
  }

  /**
   * Apply a <code>mapper</code> function on this future.<p>
   * <p>
   * When this future succeeds, the <code>mapper</code> will be called with the completed value and this mapper
   * returns a value. This value will complete the future returned by this method call.<p>
   * <p>
   * If the <code>mapper</code> throws an exception, the returned future will be failed with this exception.<p>
   * <p>
   * When this future fails, the failure will be propagated to the returned future and the <code>mapper</code>
   * will not be called.
   * @param mapper the mapper function
   * @return the mapped future
   */
  public <U> io.vertx.rxjava.core.Future<U> map(Function<T, U> mapper) { 
    io.vertx.rxjava.core.Future<U> ret = io.vertx.rxjava.core.Future.newInstance((io.vertx.core.Future)delegate.map(new Function<T,U>() {
      public U apply(T arg) {
        U ret = mapper.apply((T)__typeArg_0.wrap(arg));
        return ret;
      }
    }), TypeArg.unknown());
    return ret;
  }

  /**
   * Map the result of a future to a specific <code>value</code>.<p>
   * <p>
   * When this future succeeds, this <code>value</code> will complete the future returned by this method call.<p>
   * <p>
   * When this future fails, the failure will be propagated to the returned future.
   * @param value the value that eventually completes the mapped future
   * @return the mapped future
   */
  public <V> io.vertx.rxjava.core.Future<V> map(V value) { 
    io.vertx.rxjava.core.Future<V> ret = io.vertx.rxjava.core.Future.newInstance((io.vertx.core.Future)delegate.map(value), TypeArg.unknown());
    return ret;
  }

  /**
   * Map the result of a future to <code>null</code>.<p>
   * <p>
   * This is a conveniency for <code>future.map((T) null)</code> or <code>future.map((Void) null)</code>.<p>
   * <p>
   * When this future succeeds, <code>null</code> will complete the future returned by this method call.<p>
   * <p>
   * When this future fails, the failure will be propagated to the returned future.
   * @return the mapped future
   */
  public <V> io.vertx.rxjava.core.Future<V> mapEmpty() { 
    io.vertx.rxjava.core.Future<V> ret = io.vertx.rxjava.core.Future.newInstance((io.vertx.core.Future)delegate.mapEmpty(), TypeArg.unknown());
    return ret;
  }

  /**
   * @return an handler completing this future
   */
  @Deprecated()
  public Handler<AsyncResult<T>> completer() { 
    if (cached_0 != null) {
      return cached_0;
    }
    Handler<AsyncResult<T>> ret = new Handler<AsyncResult<T>>() {
      public void handle(AsyncResult<T> ar) {
        if (ar.succeeded()) {
          delegate.completer().handle(io.vertx.core.Future.succeededFuture(__typeArg_0.<T>unwrap(ar.result())));
        } else {
          delegate.completer().handle(io.vertx.core.Future.failedFuture(ar.cause()));
        }
      }
    };
    cached_0 = ret;
    return ret;
  }

  /**
   * Handles a failure of this Future by returning the result of another Future.
   * If the mapper fails, then the returned future will be failed with this failure.
   * @param mapper A function which takes the exception of a failure and returns a new future.
   * @return A recovered future
   */
  public io.vertx.rxjava.core.Future<T> recover(Function<java.lang.Throwable, io.vertx.rxjava.core.Future<T>> mapper) { 
    io.vertx.rxjava.core.Future<T> ret = io.vertx.rxjava.core.Future.newInstance((io.vertx.core.Future)delegate.recover(new Function<java.lang.Throwable,io.vertx.core.Future<T>>() {
      public io.vertx.core.Future<T> apply(java.lang.Throwable arg) {
        io.vertx.rxjava.core.Future<T> ret = mapper.apply(arg);
        return ret.getDelegate();
      }
    }), __typeArg_0);
    return ret;
  }

  /**
   * Apply a <code>mapper</code> function on this future.<p>
   * <p>
   * When this future fails, the <code>mapper</code> will be called with the completed value and this mapper
   * returns a value. This value will complete the future returned by this method call.<p>
   * <p>
   * If the <code>mapper</code> throws an exception, the returned future will be failed with this exception.<p>
   * <p>
   * When this future succeeds, the result will be propagated to the returned future and the <code>mapper</code>
   * will not be called.
   * @param mapper the mapper function
   * @return the mapped future
   */
  public io.vertx.rxjava.core.Future<T> otherwise(Function<java.lang.Throwable, T> mapper) { 
    io.vertx.rxjava.core.Future<T> ret = io.vertx.rxjava.core.Future.newInstance((io.vertx.core.Future)delegate.otherwise(new Function<java.lang.Throwable,T>() {
      public T apply(java.lang.Throwable arg) {
        T ret = mapper.apply(arg);
        return __typeArg_0.<T>unwrap(ret);
      }
    }), __typeArg_0);
    return ret;
  }

  /**
   * Map the failure of a future to a specific <code>value</code>.<p>
   * <p>
   * When this future fails, this <code>value</code> will complete the future returned by this method call.<p>
   * <p>
   * When this future succeeds, the result will be propagated to the returned future.
   * @param value the value that eventually completes the mapped future
   * @return the mapped future
   */
  public io.vertx.rxjava.core.Future<T> otherwise(T value) { 
    io.vertx.rxjava.core.Future<T> ret = io.vertx.rxjava.core.Future.newInstance((io.vertx.core.Future)delegate.otherwise(__typeArg_0.<T>unwrap(value)), __typeArg_0);
    return ret;
  }

  /**
   * Map the failure of a future to <code>null</code>.<p>
   * <p>
   * This is a convenience for <code>future.otherwise((T) null)</code>.<p>
   * <p>
   * When this future fails, the <code>null</code> value will complete the future returned by this method call.<p>
   * <p>
   * When this future succeeds, the result will be propagated to the returned future.
   * @return the mapped future
   */
  public io.vertx.rxjava.core.Future<T> otherwiseEmpty() { 
    io.vertx.rxjava.core.Future<T> ret = io.vertx.rxjava.core.Future.newInstance((io.vertx.core.Future)delegate.otherwiseEmpty(), __typeArg_0);
    return ret;
  }

  private Handler<AsyncResult<T>> cached_0;
  public static <T> Future<T> newInstance(io.vertx.core.Future arg) {
    return arg != null ? new Future<T>(arg) : null;
  }

  public static <T> Future<T> newInstance(io.vertx.core.Future arg, TypeArg<T> __typeArg_T) {
    return arg != null ? new Future<T>(arg, __typeArg_T) : null;
  }

}
